PCS = function(

sourceNodeCueWeights = c(.8,.7,.6,.55),
cuePattern = c(1,1,1,-1,-1,-1,-1,1),
cueOptionWeights = "default",
optionCueWeights = "default",
optionOptionWeights = "default",
actSourceNode = 1, 
actCues = "default",
actOptions = "default",
decay = .1,
floor = -1,
ceiling = 1,
lambda = 2.9,
stability = 10^6,
rescaling = 1,
animation = 0, 
plotResults = 0,
stopScreen = 1,
subtrahendResc = .5,
factorResc = 1,
exponentResc = 1.9)

######################################################################
#################### function starts here ############################
######################################################################

{

########################
########################
########################
#### recode validities and patterns if val < .5

recodeVal = which(round(sourceNodeCueWeights, 6) < .5)

if (length(recodeVal) > 0){

sourceNodeCueWeights[recodeVal] = 1 - sourceNodeCueWeights[recodeVal]

cuePattern[recodeVal] = cuePattern[recodeVal] * -1

cuePattern[recodeVal + length(sourceNodeCueWeights)] = 
cuePattern[recodeVal + length(sourceNodeCueWeights)] * -1
}

########################
########################
########################

##### assess number of cues and number of options

numberOfCues = length(sourceNodeCueWeights)
numberOfOptions = length(cuePattern)/length(sourceNodeCueWeights)  

##### set defaults

if(cueOptionWeights[1] == "default"){
cueOptionWeights = rep(.01,numberOfCues*numberOfOptions)
}

if(optionCueWeights[1] == "default"){
optionCueWeights = cueOptionWeights
}

if(optionOptionWeights[1] == "default"){
optionOptionWeights = rep(-.2,numberOfOptions*(numberOfOptions-1))
}

if(actCues[1] == "default"){
actCues = rep(0,numberOfCues)
}

if(actOptions[1] == "default"){
actOptions = rep(0,numberOfOptions)
}

##### define variables      

loop = 1
endloop = 0 

memoryActivation = 
matrix(c(actSourceNode,actCues,actOptions),nrow=1)

memory = c(0,memoryActivation,loop)
optionsInpFromOptionsSum = numeric()      
    
##### weights cues to options

cueOptionWeights = cueOptionWeights * cuePattern

optionCueWeights = optionCueWeights * cuePattern

##### rescaling of sourceNodeCueWeights

if(rescaling==1){
signWeights = sign(sourceNodeCueWeights)
sourceNodeCueWeights = 
((abs(sourceNodeCueWeights)-subtrahendResc)*factorResc)^exponentResc
sourceNodeCueWeights = signWeights * sourceNodeCueWeights
}   

######################################################################
#################### PCS iterations start here #######################
######################################################################

while(endloop == 0){  

######################################################################
#################### input for cues ##################################
######################################################################

cuesInpFromSourceNode = actSourceNode * sourceNodeCueWeights

cuesInpFromOptions = rep(actOptions, each = numberOfCues) * 
optionCueWeights

#### input for cues translated into a matrix where each row
#### represents a cue

cuesInpFromSourceNode = 
matrix(cuesInpFromSourceNode,nrow=numberOfCues,byrow=TRUE)

cuesInpFromOptions = 
matrix(cuesInpFromOptions,nrow=numberOfCues,byrow=FALSE)

inpCues = rowSums(cbind(cuesInpFromSourceNode,cuesInpFromOptions))

######################################################################
#################### input for options ###############################
######################################################################

#### input from cues

optionsInpFromCues = actCues * cueOptionWeights

#### input between options

index = as.matrix(expand.grid(1:numberOfOptions,1:numberOfOptions))
index = index[index[,1]!=index[,2],]
index = index[,2:1]

optionsInpFromOptions =
rep(actOptions,each = numberOfOptions - 1) *  optionOptionWeights

for (i in 1 : numberOfOptions){
optionsInpFromOptionsSum[i] = 
sum(optionsInpFromOptions[index[,2]==i])}

#### input for options translated into a matrix where each row
#### represents an option

optionsInpFromCues=
matrix(optionsInpFromCues,nrow=numberOfOptions,byrow=TRUE)

inputOptions = rowSums(cbind(optionsInpFromCues,
optionsInpFromOptionsSum))

######################################################################
#################### activation t + 1 ################################
######################################################################

input = c(inpCues,inputOptions)

negativeInput = memoryActivation[loop,2:NCOL(memoryActivation)] *
(1-decay) + input *
(memoryActivation[loop,2:NCOL(memoryActivation)] - floor)

positiveInput = memoryActivation[loop,2:NCOL(memoryActivation)] *
(1-decay) + input *
(ceiling - memoryActivation[loop,2:NCOL(memoryActivation)])

presentActivation = ifelse(input<0,
negativeInput,
positiveInput
)

actCues = presentActivation[1:length(inpCues)]

actOptions =
presentActivation[(length(inpCues)+1) : length(presentActivation)]  

###### save activation

memoryActivation = rbind(memoryActivation,
c(actSourceNode,presentActivation))

######################################################################
########################### energy ###################################
######################################################################

energyOptionsOptions = numeric()

## energy options options

for(loopEnergy in 1 : length(optionOptionWeights)){

energyOpOp = 
optionOptionWeights[loopEnergy] * 
actOptions[index[loopEnergy,1]] * 
actOptions[index[loopEnergy,2]]

energyOptionsOptions = c(energyOptionsOptions,energyOpOp)
}

energy = -1*(

## energy node cues

2*sum(actSourceNode * sourceNodeCueWeights * actCues) +

## energy cues options

sum(actCues * cueOptionWeights * 
rep(actOptions, each = numberOfCues)) +

sum(actCues * optionCueWeights * 
rep(actOptions, each = numberOfCues)) +

## energy options options

sum(energyOptionsOptions)

)         

###### memory

loop = loop + 1

memory =  rbind(memory,c(energy,memoryActivation[loop,],loop))

######################################################################
################## control for termination of iterations #############
######################################################################

control=0

if (loop>10){
for (i in 1 : 10){  
if (floor(stability*(memory[loop-i,1]))==
floor(stability*(memory[loop,1]))) 
control = control + 1
}}

if (control == 10) endloop = 1

######################################################################
################## animation : node activation #######################
######################################################################

colorsPLOT = rainbow(1+numberOfCues+numberOfOptions)

if(animation == 1){

plotDATA = c(memory[loop,1],
memory[loop,3:(2+numberOfCues+numberOfOptions)])

names(plotDATA) = c("energy",paste("cue", 1 : numberOfCues),
paste("option", 1 : numberOfOptions)
) 
          
barplot(plotDATA,xlab="nodes",
ylab="activation",ylim=c(floor,ceiling + .3),
col=colorsPLOT,
main=paste("||","iteration",loop,"||","energy",
round(energy,4),"||") )

for (loopPlot in seq(floor,ceiling,.25)){
abline(h = loopPlot,lty=2,col="grey")
} 

for (loopPlot in seq(floor,ceiling,.5)){
abline(h = loopPlot,lty=2)
}
 
}

}  

##### winner

winOption = which(actOptions == max(actOptions))
if(length(winOption)==length(actOptions)){winOption=0}

##### probabilistic PCS

probsPCS = exp(lambda*actOptions)/sum(exp(lambda*actOptions))

######################################################################
################## plot: node activation #############################
######################################################################

if (plotResults == 1){ 

if(stopScreen == 1 & animation == 1){

par(ask=TRUE)

}

print("please hit ENTER for next plot")

plot(1:(loop),memory[,1],type="l",ylim=c(floor,ceiling + .4),
col=colorsPLOT[1], xlab="iteration",ylab="activation",lwd=2,
main = paste("||","iteration",loop,"||","option",winOption,"||",
"energy",round(energy,4),"||")
) 

for (i in 1 : (numberOfCues + numberOfOptions)){
points(1:(loop),memory[,2+i],type="l",col=colorsPLOT[i+1],lwd=2) 
}

legend(0,ceiling + .4,cex=.85,
c("energy",paste("cue", 1:numberOfCues),
paste("option", 1:numberOfOptions)),
col=colorsPLOT,
pch=c(16,16,16,16,16,16,16),
ncol = ceiling((numberOfOptions + numberOfCues + 1)/2))

for (loopPlot in seq(floor,ceiling,.25)){
abline(h = loopPlot,lty=2,col="grey")
} 

for (loopPlot in seq(floor,ceiling,.5)){
abline(h = loopPlot,lty=2)
}

par(ask=FALSE)
}

differenceActivationOptions = max(actOptions)-min(actOptions)

######################################################################
################## output ############################################
######################################################################

colnames(memory) = c("energy","sourceNode",
paste("cue",1:numberOfCues), paste("option",1:numberOfOptions),
"iterations")

memory = list("energy, activations, iterations"=memory,
"predicted choice option" = winOption,
"number of iterations" = loop,
"difference max min activations" = differenceActivationOptions,
"predicted probability of choice for each option"=probsPCS)

return(memory)
}
